# SSH

**From any computer in the IMGW subnet**: Log in via ssh by typing either of the following in a terminal (there are two redundant login nodes, jet01 and jet02). Replace `[USERNAME]` with your own.
```bash
ssh -X [USERNAME]@srvx1.img.univie.ac.at
ssh -X [USERNAME]@srvx8.img.univie.ac.at
ssh -X [USERNAME]@jet01.img.univie.ac.at
ssh -X [USERNAME]@131.130.157.215
ssh -X [USERNAME]@jet02.img.univie.ac.at
ssh -X [USERNAME]@131.130.157.216
```
The `-X` option enables X11 forwarding via ssh, i.e., permits opening graphical windows.

Consider using a `~/.ssh/config` configuration file to allow easier access like this:
```
Host *
    ServerAliveInterval 60
    ServerAliveCountMax 2

Host srvx1
    HostName srvx1.img.univie.ac.at
    User [USERNAME]
Host srvx8
    HostName srvx8.img.univie.ac.at
    User [USERNAME]
Host jet
    HostName jet01.img.univie.ac.at
    User [USERNAME]
Host srvx2jet
    HostName jet01.img.univie.ac.at
    User [USERNAME]
    ProxyJump srvx1.img.univie.ac.at
Host login
    HostName login.univie.ac.at
    User [U:Account USERNAME]
```
and replacing `[USERNAME]` and `[U:Account USERNAME]` with your usernames. Using such a file allows to connect like this `ssh srvx1` using the correct server adress and specified username. Copy this file as well on `login.univie.ac.at` and you can use commands like this: `ssh -t login ssh jet` to connect directly to `jet` via the `login` gateway.

If you want to use ssh-keys you can also use different keys in `.ssh/config` per server with `IdentityFile ~/.ssh/id_rsa_for_server`.

**Note: If you are having trouble with your ssh-key being accepted. You need to check the selinux security policy. Because we use non standard home directories, which are not accepted by default.** Do that by running:
```bash
ls -ldZ ~/.ssh/
drwx------. 2 monkey users system_u:object_r:ssh_home_t:s0 4.0K Aug 18 21:24 .ssh/

ls -lZ ~/.ssh/
-rw-------. 1 monkey users unconfined_u:object_r:ssh_home_t:s0  1.3K Aug 17 10:58 authorized_keys
```
If you are not seeing `ssh_home_t` than you need to run `restorecon -Rv ~/.ssh/` which should update the security policy and allow for the SSH-Daemon to accept your keys.

**From eduroam**: You should be able to log in as above.

**From the outer world**: First connect to the [UniVie VPN](https://zid.univie.ac.at/en/vpn/), then start up a terminal and log in as above. Consider using [connect2vpn](connect2vpn) script to do so. Set your U:Account username as environmental variable `$VPN_USER` and run `connect2vpn`, select split (only adresses within the UNINET) or full (all connections) tunnel and watch the connection information update every 10 seconds.

If you are a guest, you can apply for a [guest u:account](https://zid.univie.ac.at/uaccount/#c11096). This will give you access to eduroam and to the VPN. Your application needs to be endorsed by a staff member, who also determines the expiration date of the account.

## SSH Authentication with keys

Find a solution [Questions - How to use ssh-key authentication?](Questions.md#q-how-to-use-ssh-key-authentication) or [Questions - How to use an ssh-agent?](Questions.md#q-how-to-use-an-ssh-agent)

## Connect Script
If you are using a terminal (Mac, Linux, WSL, ...) you can use the script [connect2jet](connect2jet) like this:
```bash
connect2jet -g [U:Account-Username]@login.univie.ac.at [Jet-Username]@jet01.img.univie.ac.at
```

There is also an option to forward a port, e.g. the VNC Port:

```bash
connect2jet -g [U:Account-Username]@login.univie.ac.at -p 5901 [Jet-Username]@jet01.img.univie.ac.at
```
which allows you to connect to `localhost:5901` and view the VNC session. Other gateway servers can be `srvx1.img.univie.ac.at`

## Tunneling

If you are connected to eduroam or you are on an external computer, you'll need to use an SSH tunnel. The instructions below refer to jet01, but you can do just the same with jet02.

On Linux, start [Remmina](https://remmina.org/), then:

*   Set "Server" to `jet01.img.univie.ac.at:[DISPLAY]` in the "Basic" tab
*   Move to the "SSH Tunnel" tab, checkout "Enable SSH Tunnel", "Same server at port 22" and specify your favourite SSH authentication method.
*   Save and connect.

On Windows, you can use either [SSVNC](http://www.karlrunge.com/x11vnc/ssvnc.html), or the combination of [Bitvise SSH Client](https://www.bitvise.com/ssh-client-download) (for the SSH tunnel) and the [RealVNC VNC Viewer](https://www.realvnc.com/en/connect/download/viewer/windows/).

Option 1: [SSVNC](http://www.karlrunge.com/x11vnc/ssvnc.html)

*   Set "VNC Host Display" to `jet01.img.univie.ac.at:[DISPLAY]`
*   Ser "Proxy/Gateway" to `[USERNAME]@jet01.img.univie.ac.at`
*   Select "Use SSH".
*   Connect. You'll be asked first for the SSH password, then for the VNC password.

Option 2: [Bitvise SSH Client](https://www.bitvise.com/ssh-client-download) and RealVNC

*   Start the Bitvise SSH client
*   Go to tab "C2S"
*   Set "Listen Interface" to `127.0.0.1`
*   Set "Listening Port" to `5900+[DISPLAY]`, e.g., `5905`
*   Set "Destination Host" to `jet01.img.univie.ac.at`
*   Set "Destination Port" to `5900+[DISPLAY]`
*   Now start VncViewer and connect to `127.0.0.1:5900+[DISPLAY]`

## SSHFS
It is possible to mount your home directory to your personal computer on Linux via `sshfs` or using of course a dedicated remote file browser like: Filezilla, Cyberduck, ...

on Linux you need to install `fuse2` and `sshfs`, the names might vary between distributions, but are all in the default repos.
```bash
# connect to srvx1 using your home directory and a srvx1 directory on your local computer
# mountserver [host] [remotedir] [localdir]
mkdir -p $HOME/srvx1
mountserver [USER]@srvx1.img.univie.ac.at /users/staff/[USER] $HOME/srvx1
```
Note the directories might vary, depending on your membership (staff, external, students).
